@@ -2,7 +2,7 @@
 /* proto_roaraudio.c: RoarAudio protocol support.
  * $Id$
  *
- *  Copyright (C) 2015      Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>
+ *  Copyright (C) 2015-2019 Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -58,6 +58,7 @@ typedef enum {
 #define HEADER_SIZE 10
 
 static int command_send(shout_t                *self,
+                        shout_connection_t     *connection,
                         shout_roar_command_t    command,
                         uint16_t                stream,
                         const void             *data,
@@ -97,14 +98,14 @@ static int command_send(shout_t                *self,
     header[8] = (datalen & 0xFF00) >> 8;
     header[9] = (datalen & 0x00FF);
 
-    shout_queue_data(&self->wqueue, header, HEADER_SIZE);
+    shout_queue_data(&connection->wqueue, header, HEADER_SIZE);
     if (datalen)
-        shout_queue_data(&self->wqueue, data, datalen);
+        shout_queue_data(&connection->wqueue, data, datalen);
 
     return SHOUTERR_SUCCESS;
 }
 
-static int shout_create_roaraudio_request_ident(shout_t *self)
+static int shout_create_roaraudio_request_ident(shout_t *self, shout_connection_t *connection)
 {
     int         ret;
     size_t      datalen;
@@ -138,24 +139,24 @@ static int shout_create_roaraudio_request_ident(shout_t *self)
     /* agent name */
     memcpy(data + 5, agent, datalen - 5);
 
-    ret = command_send(self, CMD_IDENTIFY, STREAM_NONE, data, datalen);
+    ret = command_send(self, connection, CMD_IDENTIFY, STREAM_NONE, data, datalen);
 
     free(data);
 
     return ret;
 }
 
-static int shout_create_roaraudio_request_auth(shout_t *self)
+static int shout_create_roaraudio_request_auth(shout_t *self, shout_connection_t *connection)
 {
     /* Now we send an AUTH command to the server.
      * We currently only implement the NONE type.
      * NONE type is assumed by the server if
      * we send an empty body.
      */
-    return command_send(self, CMD_AUTH, STREAM_NONE, NULL, 0);
+    return command_send(self, connection, CMD_AUTH, STREAM_NONE, NULL, 0);
 }
 
-static int shout_create_roaraudio_request_new_stream(shout_t *self)
+static int shout_create_roaraudio_request_new_stream(shout_t *self, shout_connection_t *connection)
 {
     uint32_t data[6];
 
@@ -185,10 +186,10 @@ static int shout_create_roaraudio_request_new_stream(shout_t *self)
     data[4] = htonl(2);
     data[5] = htonl(0x0010);  /* we assume Ogg/Vorbis for now. */
 
-    return command_send(self, CMD_NEW_STREAM, STREAM_NONE, data, 24);
+    return command_send(self, connection, CMD_NEW_STREAM, STREAM_NONE, data, 24);
 }
 
-static int shout_create_roaraudio_request_exec(shout_t *self)
+static int shout_create_roaraudio_request_exec(shout_t *self, shout_connection_t *connection)
 {
     /* Last an EXEC_STREAM command should be sent to open
      * an IO channel for the new stream.
@@ -196,36 +197,47 @@ static int shout_create_roaraudio_request_exec(shout_t *self)
      * after that. This very much like with SOURCE requests.
      * so no hard deal to intigrate.
      */
-    return command_send(self, CMD_EXEC_STREAM, self->protocol_extra, NULL, 0);
+    return command_send(self, connection, CMD_EXEC_STREAM, connection->protocol_extra.si, NULL, 0);
 }
 
-int shout_create_roaraudio_request(shout_t *self)
+static shout_connection_return_state_t shout_create_roaraudio_request(shout_t *self, shout_connection_t *connection)
 {
-    switch ((shout_roar_protocol_state_t)self->protocol_state) {
+    int ret;
+
+    switch ((shout_roar_protocol_state_t)connection->current_protocol_state) {
     case STATE_IDENT:
-        return shout_create_roaraudio_request_ident(self);
+        ret = shout_create_roaraudio_request_ident(self, connection);
         break;
     case STATE_AUTH:
-        return shout_create_roaraudio_request_auth(self);
+        ret = shout_create_roaraudio_request_auth(self, connection);
         break;
     case STATE_NEW_STREAM:
-        return shout_create_roaraudio_request_new_stream(self);
+        ret = shout_create_roaraudio_request_new_stream(self, connection);
         break;
     case STATE_EXEC:
-        return shout_create_roaraudio_request_exec(self);
+        ret = shout_create_roaraudio_request_exec(self, connection);
+        break;
+    default:
+        ret = SHOUTERR_INSANE;
         break;
     }
 
-    return SHOUTERR_INSANE;
+    shout_connection_set_error(connection, ret);
+    return ret == SHOUTERR_SUCCESS ? SHOUT_RS_DONE : SHOUT_RS_ERROR;
 }
 
-int shout_get_roaraudio_response(shout_t *self)
+static shout_connection_return_state_t shout_get_roaraudio_response(shout_t *self, shout_connection_t *connection)
 {
     shout_buf_t   *queue;
     size_t         total_len = 0;
     uint8_t        header[HEADER_SIZE];
 
-    for (queue = self->rqueue.head; queue; queue = queue->next) {
+    if (!connection->rqueue.len) {
+        shout_connection_set_error(connection, SHOUTERR_SOCKET);
+        return SHOUT_RS_ERROR;
+    }
+
+    for (queue = connection->rqueue.head; queue; queue = queue->next) {
         if (total_len < 10)
             memcpy(header + total_len, queue->data, queue->len > (HEADER_SIZE - total_len) ? (HEADER_SIZE - total_len) : queue->len);
         total_len += queue->len;
@@ -233,7 +245,7 @@ int shout_get_roaraudio_response(shout_t *self)
 
     /* the header alone has 10 bytes. */
     if (total_len < HEADER_SIZE)
-        return SHOUTERR_BUSY;
+        return SHOUT_RS_NOTNOW;
 
     /* ok. we got a header.
      * Now find the body length ("data length") bytes
@@ -242,14 +254,16 @@ int shout_get_roaraudio_response(shout_t *self)
      * not support.
      */
 
-    if (header[8] || header[9])
-        return SHOUTERR_UNSUPPORTED;
+    if (header[8] || header[9]) {
+        shout_connection_set_error(connection, SHOUTERR_UNSUPPORTED);
+        return SHOUT_RS_ERROR;
+    }
 
     /* Hey, we got a response. */
-    return SHOUTERR_SUCCESS;
+    return SHOUT_RS_DONE;
 }
 
-int shout_parse_roaraudio_response(shout_t *self)
+static shout_connection_return_state_t shout_parse_roaraudio_response(shout_t *self, shout_connection_t *connection)
 {
     char *data = NULL;
     uint8_t header[HEADER_SIZE];
@@ -262,46 +276,62 @@ int shout_parse_roaraudio_response(shout_t *self)
      * "data length" is already checked by shout_get_roaraudio_response().
      */
 
-    if (shout_queue_collect(self->rqueue.head, &data) != HEADER_SIZE) {
+    if (shout_queue_collect(connection->rqueue.head, &data) != HEADER_SIZE) {
         free(data);
-        return SHOUTERR_INSANE;
+        shout_connection_set_error(connection, SHOUTERR_INSANE);
+        return SHOUT_RS_ERROR;
     }
-    shout_queue_free(&self->rqueue);
+    shout_queue_free(&connection->rqueue);
     memcpy(header, data, HEADER_SIZE);
     free(data);
 
     /* check version */
-    if (header[0] != 0)
-        return SHOUTERR_UNSUPPORTED;
+    if (header[0] != 0) {
+        shout_connection_set_error(connection, SHOUTERR_UNSUPPORTED);
+        return SHOUT_RS_ERROR;
+    }
 
     /* have we got a positive response? */
-    if (header[1] != CMD_OK)
-        return SHOUTERR_NOLOGIN;
+    if (header[1] != CMD_OK) {
+        shout_connection_set_error(connection, SHOUTERR_NOLOGIN);
+        return SHOUT_RS_ERROR;
+    }
 
-    switch ((shout_roar_protocol_state_t)self->protocol_state) {
+    switch ((shout_roar_protocol_state_t)connection->current_protocol_state) {
         case STATE_IDENT:
-            self->protocol_state = STATE_AUTH;
+            connection->current_protocol_state = STATE_AUTH;
+            connection->server_caps |= LIBSHOUT_CAP_GOTCAPS;
         break;
 
         case STATE_AUTH:
-            self->protocol_state = STATE_NEW_STREAM;
+            connection->current_protocol_state = STATE_NEW_STREAM;
         break;
 
         case STATE_NEW_STREAM:
-            self->protocol_extra = (((unsigned int)header[2]) << 8) | (unsigned int)header[3];
-            self->protocol_state = STATE_EXEC;
+            connection->protocol_extra.si = (((unsigned int)header[2]) << 8) | (unsigned int)header[3];
+            connection->current_protocol_state = STATE_EXEC;
         break;
 
         case STATE_EXEC:
             /* ok. everything worked. Continue normally! */
-            return SHOUTERR_SUCCESS;
+            connection->current_message_state = SHOUT_MSGSTATE_SENDING1;
+            connection->target_message_state = SHOUT_MSGSTATE_WAITING1;
+            return SHOUT_RS_DONE;
         break;
 
         default:
-            return SHOUTERR_INSANE;
+            shout_connection_set_error(connection, SHOUTERR_INSANE);
+            return SHOUT_RS_ERROR;
         break;
     }
 
-    self->state = SHOUT_STATE_REQ_CREATION;
-    return SHOUTERR_RETRY;
+    connection->current_message_state = SHOUT_MSGSTATE_CREATING0;
+    return SHOUT_RS_DONE;
 }
+
+static const shout_protocol_impl_t shout_roaraudio_impl_real = {
+    .msg_create = shout_create_roaraudio_request,
+    .msg_get = shout_get_roaraudio_response,
+    .msg_parse = shout_parse_roaraudio_response
+};
+const shout_protocol_impl_t *shout_roaraudio_impl = &shout_roaraudio_impl_real;
